import { select,
         csv,
         scaleLinear,
         scaleTime,
         scaleOrdinal,
         extent,
         min,
         max,
         axisLeft,
         axisBottom,
         line,
         curveBasis,
         mouse,
         nest,
         schemeCategory10,
         descending
       } from "d3"
import { colorLegend } from './colorLegend';
const svg = select('svg');


const width = +svg.attr('width');
const height = +svg.attr("height");
const xLegend = 'Time'
const yLegend = 'Temperature'
const title = 'A week of temperature'
let data;
let selected = null;

const setSelected = id => {
	selected = id;
  console.log(selected)   
  render(selected);
}



const render = (selected) => {
  const xValue = d => d.timestamp
  const yValue = d => d.temperature 
  const colorValue = d => d.city
  const margin = { top:50,
                   right: 240,
                   bottom: 100,
                   left: 100
                 };
  const innerWidth = width - margin.left - margin.right
  const innerHeight = height - margin.top - margin.bottom
  const lastYValue = d => yValue(d.values[d.values.length - 1])
  const nestedData = nest()
  	.key(colorValue)
  	.entries(data)
  	.sort((a, b) => descending(lastYValue(a), lastYValue(b)));
  
  const xScale = scaleTime()
  	.domain(extent(data, xValue))
  	.range([0, innerWidth]);
  const yScale = scaleLinear()
  	.domain(extent(data, yValue))
  	.range([innerHeight, 0])
  	.nice();
  const colorScale = scaleOrdinal(schemeCategory10)
  	.domain(nestedData.map(d => d.key))
  
  const gUpdate = svg.selectAll('g').data([null])
  const gEnter = gUpdate.enter().append('g')
  	.attr('transform', `translate(${margin.left}, ${margin.top})`);
	const g = gUpdate.merge(gEnter)
  
  const xAxis = axisBottom(xScale)
  	.ticks(8)
  	.tickSize(-innerHeight)
  	.tickPadding(15);
  
  const yAxis = axisLeft(yScale)
  	.tickSize(-innerWidth)
  	.tickPadding(10);
  
  const yAxisG = g.selectAll('.y-axis').data([null]).enter()
  	.append('g').attr('class', 'y-axis')
   	 .call(yAxis);
  yAxisG.select('.domain').remove();
  
  const xAxisG = g.selectAll('.x-axis').data([null]).enter()
  	.append('g').attr('class', 'x-axis')
  		.call(xAxis)
  			.attr('transform', `translate(0, ${innerHeight})`);
  xAxisG.select('.domain').remove();
  xAxisG.append('text')
  	.attr('class', 'xLegend')
    .text(xLegend)
  	.attr('fill', 'black')
  	.attr('x', `${innerWidth / 2}`)
  	.attr('y', `${50}`);
  
  yAxisG.append('text')
  	.attr('class', 'yLegend')
    .text(yLegend)
  	.attr('transform', 'rotate(-90)')
  	.attr('fill', 'black')
  	.attr('x', -innerHeight / 2)
  	.attr('text-anchor','middle')
  	.attr('y', -50);
  
  const lineGenerator = line()
  	.x(d => xScale(xValue(d)))
  	.y(d => yScale(yValue(d)))
  	.curve(curveBasis);
  

  g.selectAll('.line-path').data(nestedData)
    .attr('stroke-opacity', d => d.key === selected || !selected
            ? 1
           	: 0.2)
    .enter()
      .append('path')
        .attr('class', 'line-path')
        .attr('stroke', d =>colorScale(d.key))
        .attr('d', d => lineGenerator(d.values))
	g.selectAll('.line-path').data(nestedData)
    .on('click',d => selected === d.key
      ? setSelected(null)
      : setSelected(d.key))

  
  xAxisG.append('text')
  	.attr('class', 'title')
  	.attr('y', -innerHeight - 5)
  	.attr('x', `${innerWidth / 2}`)
  	.text(title);
  
  const legendG = svg.selectAll('.legend').data([null])
  const legendGEnter = legendG.enter().append('g')
  	.attr('class', 'legend')
  	.attr('transform', `translate(${innerWidth+margin.left+50}, ${margin.top+20})`)
  legendG.merge(legendGEnter)
    .call(colorLegend, {
      colorScale,
      circleRadius: 10,
      spacing: 30,
      textOffset: 20,
    	setSelected,
    	selected
    });

  
  // const tooltip = select('body').append('div')
  // 	.attr('class', 'tooltip')

//   svg
//   	.on('mouseover', function(){tooltip.style('visibility', 'visible').text('asd')})
//   	.on('mousemove', function(){
//     	const x = mouse(yAxisG.node())[0]
//       if (x > 0) {
//         console.log(pathLine.node().getPointAtLength(x))
//       }
//     	console.log(mouse(yAxisG.node())[0])
    
//   	})
//   	.on("mouseout", function(){tooltip.style("visibility", "hidden").text('')})

};


csv('https://vizhub.com/curran/datasets/data-canvas-sense-your-city-one-week.csv')
  .then(newData => {
    newData.forEach(d => {
      d.temperature = +d.temperature;
      d.timestamp = new Date(d.timestamp)
    });
  // console.log(data)
  // const nested_data = nest()
  // 	.key(d => d.city)
  // 	.entries(data)
  // console.log(data)
  data = newData
	render(selected);
})